==> tivio_sdk-1.2.8.tar.gz: lib/player/tivio_player.dart <==
import 'dart:async';
import 'dart:collection';
import 'dart:io' show Platform;
import 'package:better_player/better_player.dart';
import 'package:flutter/material.dart';
import 'tivio_player_loader.dart';
import 'tivio_player_colors.dart';
import 'controls/better_player_custom_controls.dart';
import 'controls/better_player_ui_press_controls.dart';
import '../tivio.dart';
import '../player/tivio_player_input_source.dart';
import '../player/tivio_player_wrapper.dart';
import '../player/tivio_player_controller.dart';
import '../logger.dart';
import '../enums/tivio_player_state.dart';

class TivioPlayer extends StatefulWidget {
  final String playerId;
  final Tivio tivio;
  final TivioDataVideo? tivioVideoSource;
  final TivioPlayerController tivioPlayerController;
  final TivioPlayerColors tivioPlayerColors;
  final bool startInFullscreen;

  TivioPlayer({
    required this.playerId,
    required this.tivio,
    this.tivioVideoSource,
    this.startInFullscreen = false,
    tivioPlayerController,
    tivioPlayerColors,
  })  : tivioPlayerController = tivioPlayerController ?? TivioPlayerController(),
        tivioPlayerColors = tivioPlayerColors ?? TivioPlayerColors();

  @override
  _TivioPlayerState createState() => _TivioPlayerState();
}

class _TivioPlayerState extends State<TivioPlayer> {
  final StreamController<BetterPlayerEvent> playerEventStream = StreamController.broadcast();
  final StreamController<TivioPlayerInputSource> sourceStream = StreamController.broadcast();
  final Logger logger = Logger();
  late TivioPlayerWrapper tivioPlayerWrapper;
  late BetterPlayerController betterPlayerController;
  bool isBetterPlayerInitialized = false;
  int? seekToAfterInitialization;
  TivioPlayerInputSource? currentSource;
  Map<String, BetterPlayerDataSource> sources = Map();
  var durationHistory = Queue<Duration>();

  @override
  void initState() {
    super.initState();
    synchronousInitState();
  }

  @override
  Widget build(BuildContext context) {
    return Container(
      color: Colors.black,
      child: (isBetterPlayerInitialized)
          ? BetterPlayer(controller: betterPlayerController)
          : TivioPlayerLoader(fullscreen: widget.startInFullscreen),
    );
  }

  @override
  void dispose() {
    playerEventStream.close();
    sourceStream.close();
    super.dispose();
  }

  /// Public

  openAndPlay(TivioDataVideo tivioDataVideo) {
    if (tivioDataVideo.isPlaylist) {
      throw Exception('Playlists are currently unsupported');
    }

    tivioPlayerWrapper.openAndPlay(TivioPlayerVodSource(
      id: tivioDataVideo.id,
      uri: tivioDataVideo.url,
      name: tivioDataVideo.name,
      description: '',
    ));
  }

  /// Player wrapper integration

  playerPlay(payload) {
    betterPlayerController.play();
  }

  playerPause(payload) {
    betterPlayerController.pause();
  }

  playerSeekTo(Duration time) {
    betterPlayerController.postEvent(
      BetterPlayerEvent(
        BetterPlayerEventType.progress,
        parameters: {
          'progress': time,
        },
      ),
    );

    return betterPlayerController.seekTo(time);
  }

  playerOpenAndPlay(TivioPlayerInputSource source, {int? currentTime}) async {
    logger.verbose('Open and play source id: ${source.id}, type: ${source.type}, will seek to: $currentTime');

    seekToAfterInitialization = currentTime;
    currentSource = source;
    durationHistory.clear();

    sourceStream.add(source);
    betterPlayerController.setupDataSource(createPlayerSource(source));
  }

  /// Private

  skipAd() {
    tivioPlayerWrapper.skipAd();
  }

  synchronousInitState() async {
    await initTivioPlayerWrapper();
    initTivioPlayerWrapperController();
    initBetterPlayerController();

    if (this.widget.tivioVideoSource != null) {
      openAndPlay(this.widget.tivioVideoSource!);
    }
  }

  initTivioPlayerWrapper() async {
    tivioPlayerWrapper = await widget.tivio.getPlayer(widget.playerId);

    await tivioPlayerWrapper.register(TivioPlayerWrapperController(
      playerPlay,
      playerPause,
      playerSeekTo,
      playerOpenAndPlay,
    ));
  }

  initTivioPlayerWrapperController() {
    widget.tivioPlayerController.play = tivioPlayerWrapper.play;
    widget.tivioPlayerController.pause = tivioPlayerWrapper.pause;
    widget.tivioPlayerController.openAndPlay = this.openAndPlay;
    widget.tivioPlayerController.seekTo = tivioPlayerWrapper.seekTo;
  }

  initBetterPlayerController() {
    var config = BetterPlayerConfiguration(
      aspectRatio: 16 / 9,
      fit: BoxFit.fill,
      autoPlay: true,
      autoDispose: true,
      fullScreenByDefault: widget.startInFullscreen,
      eventListener: this.handleBetterPlayerEvent,
      controlsConfiguration: BetterPlayerControlsConfiguration(
        playerTheme: BetterPlayerTheme.custom,
        customControlsBuilder: (controller) => BetterPlayerCustomControls(
          initSource: currentSource!,
          sourceStream: sourceStream,
          controller: controller,
          eventPlayerStream: playerEventStream,
          tivioPlayerColors: widget.tivioPlayerColors,
          betterPlayerUiPressControls: BetterPlayerUiPressControls(
            onPlayPressed: () => widget.tivioPlayerController.play(),
            onPausePressed: () => widget.tivioPlayerController.pause(),
            onForwardPressed: () => tivioPlayerWrapper.seekByStep(15000),
            onBackwardPressed: () => tivioPlayerWrapper.seekByStep(-15000),
            onEnterFullscreenPressed: () => controller.enterFullScreen(),
            onExitFullscreenPressed: () => controller.exitFullScreen(),
            onSeekingStarted: () => widget.tivioPlayerController.pause(),
            onSeekingEnded: (Duration position) => tivioPlayerWrapper.seekTo(position.inMilliseconds),
          ),
          handleSkipAd: () => skipAd(),
        ),
      ),
    );
    var controller = BetterPlayerController(config);

    setState(() {
      betterPlayerController = controller;
      isBetterPlayerInitialized = true;
    });
  }

  BetterPlayerDataSource createPlayerSource(TivioPlayerInputSource tivioPlayerInputSource) {
    switch (tivioPlayerInputSource.type) {
      case TivioPlayerWrapperInputSourceType.Vod:
        if (tivioPlayerInputSource.id != null && sources[tivioPlayerInputSource.id!] != null) {
          return sources[tivioPlayerInputSource.id!]!;
        }

        var source = BetterPlayerDataSource(
          BetterPlayerDataSourceType.network,
          tivioPlayerInputSource.uri!,
          cacheConfiguration: BetterPlayerCacheConfiguration(
            useCache: checkIfSourceCanBeCached(),
            preCacheSize: 10 * 1024 * 1024,
            maxCacheSize: 10 * 1024 * 1024,
            maxCacheFileSize: 10 * 1024 * 1024,
          ),
        );

        if (tivioPlayerInputSource.id != null) {
          sources[tivioPlayerInputSource.id!] = source;
        }
        return source;
      case TivioPlayerWrapperInputSourceType.Advertisement:
        return BetterPlayerDataSource(
          BetterPlayerDataSourceType.network,
          tivioPlayerInputSource.uri!,
        );
      default:
        throw Exception('Unsupported type ${tivioPlayerInputSource.type} in createPlayerSource');
    }
  }

  /// There is a bug with cache on ios
  /// https://github.com/jhomlala/betterplayer/issues/461
  checkIfSourceCanBeCached() {
    return (!Platform.isIOS);
  }

  /// On iOS device with slow CPU the player may be stuck trying to play unbuffered part of the video
  /// Player can be unstuck by pressing pause and play
  unstuckPlayer(Duration nextDuration) async {
    if (!betterPlayerController.isPlaying()! || nextDuration.inMilliseconds == 0) {
      return;
    }

    if (durationHistory.length >= 10 && durationHistory.every((element) => (element.compareTo(nextDuration) == 0))) {
      logger.warning('Player was stuck');
      await betterPlayerController.pause();
      await betterPlayerController.play();
    }

    if (durationHistory.length >= 10) {
      durationHistory.removeFirst();
    }
    durationHistory.add(nextDuration);
  }

  handleBetterPlayerEvent(BetterPlayerEvent event) {
    playerEventStream.add(event);

    switch (event.betterPlayerEventType) {
      case BetterPlayerEventType.bufferingEnd:
        if (betterPlayerController.isVideoInitialized()! && seekToAfterInitialization != null) {
          playerSeekTo(Duration(milliseconds: seekToAfterInitialization!));
          seekToAfterInitialization = null;
        }
        break;
      case BetterPlayerEventType.play:
        tivioPlayerWrapper.onStateChanged(TivioPlayerState.playing);
        break;
      case BetterPlayerEventType.pause:
        tivioPlayerWrapper.onStateChanged(TivioPlayerState.paused);
        break;
      case BetterPlayerEventType.progress:
        unstuckPlayer(event.parameters!['progress']);
        tivioPlayerWrapper.onTimeChanged(event.parameters!['progress']);
        break;
      case BetterPlayerEventType.seekTo:
        tivioPlayerWrapper.onTimeChanged(event.parameters!['duration']);
        break;
      case BetterPlayerEventType.finished:
        tivioPlayerWrapper.onPlaybackEnded();
        break;
      default:
        break;
    }
  }
}

==> tivio_sdk-1.3.0.tar.gz: lib/player/tivio_player.dart <==
import 'dart:async';
import 'dart:collection';
import 'dart:io' show Platform;
import 'package:better_player/better_player.dart';
import 'package:flutter/material.dart';
import 'tivio_player_loader.dart';
import 'tivio_player_colors.dart';
import 'tivio_player_layout.dart';
import 'controls/better_player_custom_controls.dart';
import 'controls/better_player_ui_press_controls.dart';
import '../tivio.dart';
import '../player/tivio_player_input_source.dart';
import '../player/tivio_player_wrapper.dart';
import '../player/tivio_player_controller.dart';
import '../logger.dart';
import '../enums/tivio_player_state.dart';


class TivioPlayer extends StatefulWidget {
  final String playerId;
  final Tivio tivio;
  final TivioDataVideo? tivioVideoSource;
  final TivioPlayerController tivioPlayerController;
  final TivioPlayerColors tivioPlayerColors;
  final TivioPlayerLayout tivioPlayerLayout;
  final bool startInFullscreen;

  TivioPlayer({
    required this.playerId,
    required this.tivio,
    this.tivioVideoSource,
    this.startInFullscreen = false,
    tivioPlayerController,
    tivioPlayerColors,
    tivioPlayerLayout,
  })  : tivioPlayerController = tivioPlayerController ?? TivioPlayerController(),
        tivioPlayerColors = tivioPlayerColors ?? TivioPlayerColors(),
        tivioPlayerLayout = tivioPlayerLayout ?? TivioPlayerLayout();

  @override
  _TivioPlayerState createState() => _TivioPlayerState();
}

class _TivioPlayerState extends State<TivioPlayer> {
  final StreamController<BetterPlayerEvent> playerEventStream = StreamController.broadcast();
  final StreamController<TivioPlayerInputSource> sourceStream = StreamController.broadcast();
  final Logger logger = Logger();
  late TivioPlayerWrapper tivioPlayerWrapper;
  late BetterPlayerController betterPlayerController;
  bool isBetterPlayerInitialized = false;
  int? seekToAfterInitialization;
  TivioPlayerInputSource? currentSource;
  Map<String, BetterPlayerDataSource> sources = Map();
  var durationHistory = Queue<Duration>();

  @override
  void initState() {
    super.initState();
    synchronousInitState();
  }

  @override
  Widget build(BuildContext context) {
    return Container(
      color: Colors.black,
      child: (isBetterPlayerInitialized)
          ? BetterPlayer(controller: betterPlayerController)
          : TivioPlayerLoader(fullscreen: widget.startInFullscreen),
    );
  }

  @override
  void dispose() {
    playerEventStream.close();
    sourceStream.close();
    super.dispose();
  }

  /// Public

  openAndPlay(TivioDataVideo tivioDataVideo) {
    if (tivioDataVideo.isPlaylist) {
      throw Exception('Playlists are currently unsupported');
    }

    tivioPlayerWrapper.openAndPlay(TivioPlayerVodSource(
      id: tivioDataVideo.id,
      uri: tivioDataVideo.url,
      name: tivioDataVideo.name,
      description: '',
    ));
  }

  /// Player wrapper integration

  playerPlay(payload) {
    betterPlayerController.play();
  }

  playerPause(payload) {
    betterPlayerController.pause();
  }

  playerSeekTo(Duration time) {
    betterPlayerController.postEvent(
      BetterPlayerEvent(
        BetterPlayerEventType.progress,
        parameters: {
          'progress': time,
        },
      ),
    );

    return betterPlayerController.seekTo(time);
  }

  playerOpenAndPlay(TivioPlayerInputSource source, {int? currentTime}) async {
    logger.verbose('Open and play source id: ${source.id}, type: ${source.type}, will seek to: $currentTime');

    seekToAfterInitialization = currentTime;
    currentSource = source;
    durationHistory.clear();

    sourceStream.add(source);
    betterPlayerController.setupDataSource(createPlayerSource(source));
  }

  /// Private

  skipAd() {
    tivioPlayerWrapper.skipAd();
  }

  synchronousInitState() async {
    await initTivioPlayerWrapper();
    initTivioPlayerWrapperController();
    initBetterPlayerController();

    if (this.widget.tivioVideoSource != null) {
      openAndPlay(this.widget.tivioVideoSource!);
    }
  }

  initTivioPlayerWrapper() async {
    tivioPlayerWrapper = await widget.tivio.getPlayer(widget.playerId);

    await tivioPlayerWrapper.register(TivioPlayerWrapperController(
      playerPlay,
      playerPause,
      playerSeekTo,
      playerOpenAndPlay,
    ));
  }

  initTivioPlayerWrapperController() {
    widget.tivioPlayerController.play = tivioPlayerWrapper.play;
    widget.tivioPlayerController.pause = tivioPlayerWrapper.pause;
    widget.tivioPlayerController.openAndPlay = this.openAndPlay;
    widget.tivioPlayerController.seekTo = tivioPlayerWrapper.seekTo;
  }

  initBetterPlayerController() {
    var config = BetterPlayerConfiguration(
      aspectRatio: 16 / 9,
      fit: BoxFit.fill,
      autoPlay: true,
      autoDispose: true,
      fullScreenByDefault: widget.startInFullscreen,
      eventListener: this.handleBetterPlayerEvent,
      controlsConfiguration: BetterPlayerControlsConfiguration(
        playerTheme: BetterPlayerTheme.custom,
        customControlsBuilder: (controller) => BetterPlayerCustomControls(
          initSource: currentSource!,
          sourceStream: sourceStream,
          controller: controller,
          eventPlayerStream: playerEventStream,
          tivioPlayerColors: widget.tivioPlayerColors,
          tivioPlayerLayout: widget.tivioPlayerLayout,
          betterPlayerUiPressControls: BetterPlayerUiPressControls(
            onPlayPressed: () => widget.tivioPlayerController.play(),
            onPausePressed: () => widget.tivioPlayerController.pause(),
            onForwardPressed: () => tivioPlayerWrapper.seekByStep(15000),
            onBackwardPressed: () => tivioPlayerWrapper.seekByStep(-15000),
            onEnterFullscreenPressed: () => controller.enterFullScreen(),
            onExitFullscreenPressed: () => controller.exitFullScreen(),
            onSeekingStarted: () => widget.tivioPlayerController.pause(),
            onSeekingEnded: (Duration position) => tivioPlayerWrapper.seekTo(position.inMilliseconds),
          ),
          handleSkipAd: () => skipAd(),
        ),
      ),
    );
    var controller = BetterPlayerController(config);

    setState(() {
      betterPlayerController = controller;
      isBetterPlayerInitialized = true;
    });
  }

  BetterPlayerDataSource createPlayerSource(TivioPlayerInputSource tivioPlayerInputSource) {
    switch (tivioPlayerInputSource.type) {
      case TivioPlayerWrapperInputSourceType.Vod:
        if (tivioPlayerInputSource.id != null && sources[tivioPlayerInputSource.id!] != null) {
          return sources[tivioPlayerInputSource.id!]!;
        }

        var source = BetterPlayerDataSource(
          BetterPlayerDataSourceType.network,
          tivioPlayerInputSource.uri!,
          cacheConfiguration: BetterPlayerCacheConfiguration(
            useCache: checkIfSourceCanBeCached(),
            preCacheSize: 10 * 1024 * 1024,
            maxCacheSize: 10 * 1024 * 1024,
            maxCacheFileSize: 10 * 1024 * 1024,
          ),
        );

        if (tivioPlayerInputSource.id != null) {
          sources[tivioPlayerInputSource.id!] = source;
        }
        return source;
      case TivioPlayerWrapperInputSourceType.Advertisement:
        return BetterPlayerDataSource(
          BetterPlayerDataSourceType.network,
          tivioPlayerInputSource.uri!,
        );
      default:
        throw Exception('Unsupported type ${tivioPlayerInputSource.type} in createPlayerSource');
    }
  }

  /// There is a bug with cache on ios
  /// https://github.com/jhomlala/betterplayer/issues/461
  checkIfSourceCanBeCached() {
    return (!Platform.isIOS);
  }

  /// On iOS device with slow CPU the player may be stuck trying to play unbuffered part of the video
  /// Player can be unstuck by pressing pause and play
  unstuckPlayer(Duration nextDuration) async {
    if (!betterPlayerController.isPlaying()! || nextDuration.inMilliseconds == 0) {
      return;
    }

    if (durationHistory.length >= 10 && durationHistory.every((element) => (element.compareTo(nextDuration) == 0))) {
      logger.warning('Player was stuck');
      await betterPlayerController.pause();
      await betterPlayerController.play();
    }

    if (durationHistory.length >= 10) {
      durationHistory.removeFirst();
    }
    durationHistory.add(nextDuration);
  }

  handleBetterPlayerEvent(BetterPlayerEvent event) {
    playerEventStream.add(event);

    switch (event.betterPlayerEventType) {
      case BetterPlayerEventType.bufferingEnd:
        if (betterPlayerController.isVideoInitialized()! && seekToAfterInitialization != null) {
          playerSeekTo(Duration(milliseconds: seekToAfterInitialization!));
          seekToAfterInitialization = null;
        }
        break;
      case BetterPlayerEventType.play:
        tivioPlayerWrapper.onStateChanged(TivioPlayerState.playing);
        break;
      case BetterPlayerEventType.pause:
        tivioPlayerWrapper.onStateChanged(TivioPlayerState.paused);
        break;
      case BetterPlayerEventType.progress:
        unstuckPlayer(event.parameters!['progress']);
        tivioPlayerWrapper.onTimeChanged(event.parameters!['progress']);
        break;
      case BetterPlayerEventType.seekTo:
        tivioPlayerWrapper.onTimeChanged(event.parameters!['duration']);
        break;
      case BetterPlayerEventType.finished:
        tivioPlayerWrapper.onPlaybackEnded();
        break;
      default:
        break;
    }
  }
}
